@@ -1,4 +1,5 @@
 //  Copyright (c) 2006-7 John Maddock
+//  Copyright (c) 2021 Matt Borland
 //  Use, modification and distribution are subject to the
 //  Boost Software License, Version 1.0. (See accompanying file
 //  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -10,21 +11,73 @@
 #pragma once
 #endif
 
+#include <boost/math/tools/is_standalone.hpp>
+
+#ifndef BOOST_MATH_STANDALONE
 #include <boost/config.hpp>
-#include <boost/predef/architecture/x86.h>
-#include <boost/cstdint.hpp> // for boost::uintmax_t
-#include <boost/detail/workaround.hpp>
-#include <boost/type_traits/is_integral.hpp>
+
+#else // Things from boost/config that are required, and easy to replicate
+
+#define BOOST_PREVENT_MACRO_SUBSTITUTION
+#define BOOST_MATH_NO_REAL_CONCEPT_TESTS
+#define BOOST_MATH_NO_DISTRIBUTION_CONCEPT_TESTS
+#define BOOST_MATH_NO_LEXICAL_CAST
+#define TEST_STD
+
+#if (__cplusplus > 201400L || _MSVC_LANG > 201400L)
+#define BOOST_CXX14_CONSTEXPR constexpr
+#else
+#define BOOST_CXX14_CONSTEXPR
+#define BOOST_NO_CXX14_CONSTEXPR
+#endif // BOOST_CXX14_CONSTEXPR
+
+#if (__cplusplus > 201700L || _MSVC_LANG > 201700L)
+#define BOOST_IF_CONSTEXPR if constexpr
+#if !__has_include(<execution>)
+#define BOOST_NO_CXX17_HDR_EXECUTION
+#endif
+#else
+#define BOOST_IF_CONSTEXPR if
+#define BOOST_NO_CXX17_IF_CONSTEXPR
+#define BOOST_NO_CXX17_HDR_EXECUTION
+#endif
+
+#define BOOST_JOIN(X, Y) BOOST_DO_JOIN(X, Y)
+#define BOOST_DO_JOIN(X, Y) BOOST_DO_JOIN2(X,Y)
+#define BOOST_DO_JOIN2(X, Y) X##Y
+
+#define BOOST_STRINGIZE(X) BOOST_DO_STRINGIZE(X)
+#define BOOST_DO_STRINGIZE(X) #X
+
+#ifdef BOOST_DISABLE_THREADS // No threads, do nothing
+// Detect thread support via STL implementation
+#elif defined(__has_include)
+#  if !__has_include(<thread>) || !__has_include(<mutex>) || !__has_include(<future>) || !__has_include(<atomic>)
+#     define BOOST_DISABLE_THREADS
+#  else
+#     define BOOST_HAS_THREADS
+#  endif 
+#else
+#  define BOOST_HAS_THREADS // The default assumption is that the machine has threads
+#endif // Thread Support
+
+#ifdef BOOST_DISABLE_THREADS
+#  define BOOST_NO_CXX11_HDR_ATOMIC
+#  define BOOST_NO_CXX11_HDR_FUTURE
+#  define BOOST_NO_CXX11_HDR_THREAD
+#  define BOOST_NO_CXX11_THREAD_LOCAL
+#endif // BOOST_DISABLE_THREADS
+
+#endif // BOOST_MATH_STANDALONE
+
 #include <algorithm>  // for min and max
-#include <boost/config/no_tr1/cmath.hpp>
+#include <limits>
+#include <cmath>
 #include <climits>
 #include <cfloat>
 #if (defined(macintosh) || defined(__APPLE__) || defined(__APPLE_CC__))
 #  include <math.h>
 #endif
-#ifndef BOOST_NO_LIMITS_COMPILE_TIME_CONSTANTS
-#  include <limits>
-#endif
 
 #include <boost/math/tools/user.hpp>
 
@@ -33,23 +86,15 @@
    && !defined(BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS)
 #  define BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
 #endif
-#if BOOST_WORKAROUND(BOOST_BORLANDC, BOOST_TESTED_AT(0x582))
-//
-// Borland post 5.8.2 uses Dinkumware's std C lib which
-// doesn't have true long double precision.  Earlier
-// versions are problematic too:
-//
-#  define BOOST_MATH_NO_REAL_CONCEPT_TESTS
-#  define BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
-#  define BOOST_MATH_CONTROL_FP _control87(MCW_EM,MCW_EM)
-#  include <float.h>
-#endif
+
 #ifdef __IBMCPP__
 //
 // For reasons I don't understand, the tests with IMB's compiler all
 // pass at long double precision, but fail with real_concept, those tests
 // are disabled for now.  (JM 2012).
+#ifndef BOOST_MATH_NO_REAL_CONCEPT_TESTS
 #  define BOOST_MATH_NO_REAL_CONCEPT_TESTS
+#endif // BOOST_MATH_NO_REAL_CONCEPT_TESTS
 #endif
 #ifdef sun
 // Any use of __float128 in program startup code causes a segfault  (tested JM 2015, Solaris 11).
@@ -101,7 +146,7 @@
 #define BOOST_MATH_DISABLE_STD_FPCLASSIFY
 #endif
 
-#if defined(BOOST_MSVC) && !defined(_WIN32_WCE)
+#if defined(_MSC_VER) && !defined(_WIN32_WCE)
    // Better safe than sorry, our tests don't support hardware exceptions:
 #  define BOOST_MATH_CONTROL_FP _control87(MCW_EM,MCW_EM)
 #endif
@@ -126,22 +171,27 @@
 #  define BOOST_MATH_USE_C99
 #endif
 
-#if defined(__CYGWIN__) || defined(__HP_aCC) || defined(BOOST_INTEL) \
+#if defined(__CYGWIN__) || defined(__HP_aCC) || defined(__INTEL_COMPILER) \
   || defined(BOOST_NO_NATIVE_LONG_DOUBLE_FP_CLASSIFY) \
   || (defined(__GNUC__) && !defined(BOOST_MATH_USE_C99))\
   || defined(BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS)
 #  define BOOST_MATH_NO_NATIVE_LONG_DOUBLE_FP_CLASSIFY
 #endif
 
-#if BOOST_WORKAROUND(__SUNPRO_CC, <= 0x590)
+#if defined(__SUNPRO_CC) && (__SUNPRO_CC <= 0x590)
 
-#  include "boost/type.hpp"
-#  include "boost/non_type.hpp"
+namespace boost { namespace math { namespace tools { namespace detail {
+template <typename T>
+struct type {};
 
-#  define BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(t)         boost::type<t>* = 0
-#  define BOOST_MATH_EXPLICIT_TEMPLATE_TYPE_SPEC(t)    boost::type<t>*
-#  define BOOST_MATH_EXPLICIT_TEMPLATE_NON_TYPE(t, v)  boost::non_type<t, v>* = 0
-#  define BOOST_MATH_EXPLICIT_TEMPLATE_NON_TYPE_SPEC(t, v)  boost::non_type<t, v>*
+template <typename T, T n>
+struct non_type {};
+}}}} // Namespace boost, math tools, detail
+
+#  define BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(t)              boost::math::tools::detail::type<t>* = 0
+#  define BOOST_MATH_EXPLICIT_TEMPLATE_TYPE_SPEC(t)         boost::math::tools::detail::type<t>*
+#  define BOOST_MATH_EXPLICIT_TEMPLATE_NON_TYPE(t, v)       boost::math::tools::detail::non_type<t, v>* = 0
+#  define BOOST_MATH_EXPLICIT_TEMPLATE_NON_TYPE_SPEC(t, v)  boost::math::tools::detail::non_type<t, v>*
 
 #  define BOOST_MATH_APPEND_EXPLICIT_TEMPLATE_TYPE(t)         \
              , BOOST_MATH_EXPLICIT_TEMPLATE_TYPE(t)
@@ -177,31 +227,22 @@
 #  define BOOST_MATH_SMALL_CONSTANT(x) x
 #endif
 
-
-#if BOOST_WORKAROUND(BOOST_MSVC, < 1400)
-//
-// Define if constants too large for a float cause "bad"
-// values to be stored in the data, rather than infinity
-// or a suitably large value.
-//
-#  define BOOST_MATH_BUGGY_LARGE_FLOAT_CONSTANTS
-#endif
 //
 // Tune performance options for specific compilers:
 //
-#ifdef BOOST_MSVC
+#ifdef _MSC_VER
 #  define BOOST_MATH_POLY_METHOD 2
-#if BOOST_MSVC <= 1900
+#if _MSC_VER <= 1900
 #  define BOOST_MATH_RATIONAL_METHOD 1
 #else
 #  define BOOST_MATH_RATIONAL_METHOD 2
 #endif
-#if BOOST_MSVC > 1900
+#if _MSC_VER > 1900
 #  define BOOST_MATH_INT_TABLE_TYPE(RT, IT) RT
 #  define BOOST_MATH_INT_VALUE_SUFFIX(RV, SUF) RV##.0L
 #endif
 
-#elif defined(BOOST_INTEL)
+#elif defined(__INTEL_COMPILER)
 #  define BOOST_MATH_POLY_METHOD 2
 #  define BOOST_MATH_RATIONAL_METHOD 1
 
@@ -227,44 +268,12 @@
 
 #endif
 
-#if defined(BOOST_NO_LONG_LONG) && !defined(BOOST_MATH_INT_TABLE_TYPE)
-#  define BOOST_MATH_INT_TABLE_TYPE(RT, IT) RT
-#  define BOOST_MATH_INT_VALUE_SUFFIX(RV, SUF) RV##.0L
-#endif
-
-//
-// constexpr support, early GCC implementations can't cope so disable
-// constexpr for them:
-//
-#if !defined(__clang__) && defined(__GNUC__)
-#if (__GNUC__ * 100 + __GNUC_MINOR__) < 490
-#  define BOOST_MATH_DISABLE_CONSTEXPR
-#endif
-#endif
-
-#ifdef BOOST_MATH_DISABLE_CONSTEXPR
-#  define BOOST_MATH_CONSTEXPR
-#else
-#  define BOOST_MATH_CONSTEXPR BOOST_CONSTEXPR
-#endif
-
 //
 // noexcept support:
 //
-#ifndef BOOST_NO_CXX11_NOEXCEPT
-#ifndef BOOST_NO_CXX11_HDR_TYPE_TRAITS
 #include <type_traits>
-#  define BOOST_MATH_NOEXCEPT(T) noexcept(std::is_floating_point<T>::value)
-#  define BOOST_MATH_IS_FLOAT(T) (std::is_floating_point<T>::value)
-#else
-#include <boost/type_traits/is_floating_point.hpp>
-#  define BOOST_MATH_NOEXCEPT(T) noexcept(boost::is_floating_point<T>::value)
-#  define BOOST_MATH_IS_FLOAT(T) (boost::is_floating_point<T>::value)
-#endif
-#else
-#  define BOOST_MATH_NOEXCEPT(T)
-#  define BOOST_MATH_IS_FLOAT(T) false
-#endif
+#define BOOST_MATH_NOEXCEPT(T) noexcept(std::is_floating_point<T>::value)
+#define BOOST_MATH_IS_FLOAT(T) (std::is_floating_point<T>::value)
 
 //
 // The maximum order of polynomial that will be evaluated 
@@ -294,17 +303,19 @@
 //
 // And then the actual configuration:
 //
-#if (defined(_GLIBCXX_USE_FLOAT128) && !defined(__NetBSD__)) && defined(BOOST_GCC) && !defined(__STRICT_ANSI__) \
-   && !defined(BOOST_MATH_DISABLE_FLOAT128) || defined(BOOST_MATH_USE_FLOAT128)
+#if defined(BOOST_MATH_STANDALONE) && defined(_GLIBCXX_USE_FLOAT128) && defined(__GNUC__) && defined(__GNUC_MINOR__) && defined(__GNUC_PATCHLEVEL__) && !defined(__STRICT_ANSI__) \
+   && !defined(__NetBSD__) \
+   && !defined(BOOST_MATH_DISABLE_FLOAT128) && !defined(BOOST_MATH_USE_FLOAT128)
+#  define BOOST_MATH_USE_FLOAT128
+#elif defined(BOOST_HAS_FLOAT128) && !defined(BOOST_MATH_USE_FLOAT128)
+#  define BOOST_MATH_USE_FLOAT128
+#endif
+#ifdef BOOST_MATH_USE_FLOAT128
 //
 // Only enable this when the compiler really is GCC as clang and probably 
 // intel too don't support __float128 yet :-(
 //
-#ifndef BOOST_MATH_USE_FLOAT128
-#  define BOOST_MATH_USE_FLOAT128
-#endif
-
-#  if defined(BOOST_INTEL) && defined(BOOST_INTEL_CXX_VERSION) && (BOOST_INTEL_CXX_VERSION >= 1310) && defined(__GNUC__)
+#  if defined(__INTEL_COMPILER) && defined(__GNUC__)
 #    if (__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 6))
 #      define BOOST_MATH_FLOAT128_TYPE __float128
 #    endif
@@ -387,11 +398,7 @@ namespace detail{
 template <class T>
 struct is_integer_for_rounding
 {
-   static const bool value = boost::is_integral<T>::value
-#ifndef BOOST_NO_LIMITS_COMPILE_TIME_CONSTANTS
-      || (std::numeric_limits<T>::is_specialized && std::numeric_limits<T>::is_integer)
-#endif
-      ;
+   static constexpr bool value = std::is_integral<T>::value || (std::numeric_limits<T>::is_specialized && std::numeric_limits<T>::is_integer);
 };
 
 }
@@ -404,7 +411,7 @@ struct is_integer_for_rounding
 #  endif
 #endif
 
-#if ((defined(__linux__) && !defined(__UCLIBC__) && !defined(BOOST_MATH_HAVE_FIXED_GLIBC)) || defined(__QNX__) || defined(__IBMCPP__)) && !defined(BOOST_NO_FENV_H)
+#if ((defined(__linux__) && !defined(__UCLIBC__) && !defined(BOOST_MATH_HAVE_FIXED_GLIBC)) || defined(__QNX__) || defined(__IBMCPP__))
 //
 // This code was introduced in response to this glibc bug: http://sourceware.org/bugzilla/show_bug.cgi?id=2445
 // Basically powl and expl can return garbage when the result is small and certain exception flags are set
@@ -412,7 +419,7 @@ struct is_integer_for_rounding
 // Much more information in this message thread: https://groups.google.com/forum/#!topic/boost-list/ZT99wtIFlb4
 //
 
-   #include <boost/detail/fenv.hpp>
+#include <cfenv>
 
 #  ifdef FE_ALL_EXCEPT
 
@@ -460,7 +467,7 @@ namespace boost{ namespace math{
 
 #  define BOOST_MATH_INSTRUMENT_CODE(x) \
       std::cout << std::setprecision(35) << __FILE__ << ":" << __LINE__ << " " << x << std::endl;
-#  define BOOST_MATH_INSTRUMENT_VARIABLE(name) BOOST_MATH_INSTRUMENT_CODE(BOOST_STRINGIZE(name) << " = " << name)
+#  define BOOST_MATH_INSTRUMENT_VARIABLE(name) BOOST_MATH_INSTRUMENT_CODE(#name << " = " << name)
 
 #else
 
@@ -472,16 +479,17 @@ namespace boost{ namespace math{
 //
 // Thread local storage:
 //
-#if !defined(BOOST_NO_CXX11_THREAD_LOCAL) && !defined(BOOST_INTEL)
+#ifndef BOOST_DISABLE_THREADS
 #  define BOOST_MATH_THREAD_LOCAL thread_local
 #else
-#  define BOOST_MATH_THREAD_LOCAL
+#  define BOOST_MATH_THREAD_LOCAL 
 #endif
+
 //
 // Some mingw flavours have issues with thread_local and types with non-trivial destructors
 // See https://sourceforge.net/p/mingw-w64/bugs/527/
 //
-#if !defined(BOOST_NO_CXX11_THREAD_LOCAL) && (defined(__MINGW32__) || defined(__MINGW64__)) && !defined(_REENTRANT) && !defined(__clang__)
+#if (defined(__MINGW32__) && (__GNUC__ < 9) && !defined(__clang__))
 #  define BOOST_MATH_NO_THREAD_LOCAL_WITH_NON_TRIVIAL_TYPES
 #endif
 
@@ -489,7 +497,7 @@ namespace boost{ namespace math{
 //
 // Can we have constexpr tables?
 //
-#if (!defined(BOOST_NO_CXX11_HDR_ARRAY) && !defined(BOOST_NO_CXX14_CONSTEXPR)) || BOOST_WORKAROUND(BOOST_MSVC, >= 1910)
+#if (!defined(BOOST_NO_CXX14_CONSTEXPR)) || (defined(_MSC_VER) && _MSC_VER >= 1910)
 #define BOOST_MATH_HAVE_CONSTEXPR_TABLES
 #define BOOST_MATH_CONSTEXPR_TABLE_FUNCTION constexpr
 #else
